

#include "error.h"


#include "kvbuffer.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>


#ifdef WIN32
#define snprintf _snprintf
#endif


KVBuffer::KVBuffer()
:nbPairs_(0)
{
}
KVBuffer::~KVBuffer()
{
}

void KVBuffer::reset()
{
  nbPairs_ = 0;
}

bool KVBuffer::initialize(const char* ibuf)
{
  nbPairs_ = 0;

  debug("KVBuffer::initialize");

  int bufLen = strlen(ibuf);

  debug("ibuf : len=%d / ibuf='%s'", bufLen, ibuf);

  const char* first = ibuf;
  const char* last = ibuf;
  bool end = false;
  while(first[0] != 0 && !end)
  {
    last = strchr(last, '=');
    if (last == (char*)-1)
    {
      end = true;
      break;
    }
    memcpy(pairs_[nbPairs_].key_, first, last-first);
    pairs_[nbPairs_].key_[last-first] = 0;

    first = last+1;

    last = strchr(last, ';');
    if (last == (char*)0)
    {
      last = &ibuf[bufLen];
      end = true;
    }
    memcpy(pairs_[nbPairs_].value_, first, last-first);
    pairs_[nbPairs_].value_[last-first] = 0;

    debug("%d : Key='%s' / Value='%s'", nbPairs_, pairs_[nbPairs_].key_, pairs_[nbPairs_].value_);

    first = last+1;
    nbPairs_++;
  }

  debug("KVBuffer::initialize : End OK");

  return true;
}

int KVBuffer::getIntValue(const char* key) const
{
  const char* value = getValue(key);
  if (value == 0)
  {
    return -1;
  }
  return atoi(value);
}

const char* KVBuffer::getValue(const char* key) const
{
  for (int i=0;i<nbPairs_;i++)
  {
    if (strcmp(pairs_[i].key_, key) == 0)
    {
      debug("KVBuffer::getValue: key='%s' / value='%s'", key, pairs_[i].value_);
      return pairs_[i].value_;
    }

    //debug("Pair doesn't match : try='%s' / key='%s'", key, pairs_[i].key_);
  }

  debug("KVBuffer::getValue: key='%s' / value=<not found>", key);
  return 0;
}

bool KVBuffer::addKeyValue(const char* key, const char* value)
{
  debug("KVBuffer::addKeyValue : Pair=%d: key='%s' / value='%s'", nbPairs_, key, value);
  strncpy(pairs_[nbPairs_].key_, key, MAX_KEY_LEN);
  strncpy(pairs_[nbPairs_].value_, value, MAX_VALUE_LEN);
  nbPairs_++;
  return true;
}
bool KVBuffer::addKeyIntValue(const char* key, int value)
{
  debug("KVBuffer::addKeyIntValue : Pair=%d: key='%s' / value='%d'", nbPairs_, key, value);
  strncpy(pairs_[nbPairs_].key_, key, MAX_KEY_LEN);
  snprintf(pairs_[nbPairs_].value_, MAX_VALUE_LEN, "%d", value);
  nbPairs_++;
  return true;
}

bool KVBuffer::getString(char* obuf, int obuf_len) const
{
  static char tmp[2048];
  static char temp[1024];
  tmp[0] = 0;
  temp[0] = 0;
  for (int i=0;i<nbPairs_;i++)
  {
    if (i > 0)
    {
      strcat(tmp, ";");
    }
    snprintf(temp, 1024, "%s=%s", pairs_[i].key_, pairs_[i].value_);
    strncat(tmp, temp, 2048-strlen(tmp) );
  }
  strncpy(obuf, tmp, obuf_len);

  int nbytes = strlen(tmp) + 8;
  int size_of_nbytes = 0;
  if (nbytes < 9)
  {
    nbytes += 1;
  }
  else if (nbytes < 98)
  {
    nbytes += 2;
  }
  else if (nbytes < 997)
  {
    nbytes += 3;
  }
  else
  {
    nbytes += 4;
  }

  sprintf(temp, "nbytes=%d", nbytes);

  snprintf(obuf, obuf_len, "%s;%s", temp, tmp);
  return true;
}
