/*******************************************************************
 *
 * Enumerate All Processes for NT
 * Copyright (c) 2000 Jan Christian Kaldestad (ai97jck@stud.hib.no)
 * Last Modified: 12.05.2000
 * 
 *
 * Description:
 *  This program enumerates all process on the system and writes
 *	the process-name to a named pipe which is used to communicate with
 *	the nsserv.exe program.  This program is only meant to be used as
 *  a "help-process" inside the nsserv.exe.
 *
 *
 ******************************************************************/


#include <stdio.h>
#include <tchar.h>
#include <iostream.h>
#include <assert.h>
#include <pdh.h>

LPTSTR GetLastErrorText( LPTSTR lpszBuf, DWORD dwSize, DWORD dwErr )
{
    DWORD dwRet;
    LPTSTR lpszTemp = NULL;

    dwRet = FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |FORMAT_MESSAGE_ARGUMENT_ARRAY,
                           NULL,
                           dwErr,
                           LANG_NEUTRAL,
                           (LPTSTR)&lpszTemp,
                           0,
                           NULL );

    // supplied buffer is not long enough
    if ( !dwRet || ( (long)dwSize < (long)dwRet+14 ) )
        lpszBuf[0] = TEXT('\0');
    else
    {
        lpszTemp[lstrlen(lpszTemp)-2] = TEXT('\0');  //remove cr and newline character
        _stprintf( lpszBuf, TEXT("%s (0x%x)"), lpszTemp, GetLastError() );
    }

    if ( lpszTemp )
        LocalFree((HLOCAL) lpszTemp );

    return lpszBuf;
}




int main(int argc, LPTSTR argv[] ) 
{	
	PDH_STATUS	err;
	LPTSTR		CounterListBuf = NULL;
	DWORD		CounterListSize = 0;
	LPTSTR		InstanceListBuf = NULL;
	DWORD		InstanceListSize = 0;
	LPSTR		ThisInstance = NULL;
	HQUERY		hQuery;

	int			prosCount = 0;	

	HANDLE		hPipe;
	LPTSTR		lpPipename = "\\\\.\\pipe\\ProsListPipe"; 
	
	DWORD		nWritten;


	err = PdhOpenQuery (NULL, 0, &hQuery);
	assert( err == ERROR_SUCCESS);
	
	// Get bufferspace needed to build the process-array
	err = PdhEnumObjectItems( NULL,
							  NULL,
							  TEXT("Process"),
							  CounterListBuf,
							  &CounterListSize,
							  InstanceListBuf,		
							  &InstanceListSize,
							  PERF_DETAIL_WIZARD,
							  0);
	assert( err == ERROR_SUCCESS);

	// Allocate memory
	CounterListBuf  = (LPSTR) malloc((CounterListSize * sizeof(TCHAR)));
	InstanceListBuf = (LPSTR) malloc((InstanceListSize * sizeof(TCHAR)));

	// Call PdhEnumObjectItems again and build an array 
	// of all processes running on the system
	if(CounterListBuf != NULL && InstanceListBuf  != NULL) {

		err = PdhEnumObjectItems( NULL,
							  NULL,
							  TEXT("Process"),
							  CounterListBuf,
							  &CounterListSize,
							  InstanceListBuf,		
							  &InstanceListSize,
							  PERF_DETAIL_WIZARD,
							  0);
		assert( err == ERROR_SUCCESS);
		
		// Create anI/O object which we write process-names to
		hPipe = CreateFile( lpPipename,
							GENERIC_WRITE,
							FILE_SHARE_READ,
							NULL,
							OPEN_EXISTING,
							NULL,
							NULL );
    { // Block
    	char lastError[512];
    	DWORD dwErr = GetLastError();	
	    if(dwErr != ERROR_SUCCESS) 
	    {
		    GetLastErrorText(lastError, 512, dwErr);
	    }
    } // Block

		DWORD dw = PIPE_READMODE_MESSAGE | PIPE_WAIT;
		SetNamedPipeHandleState( hPipe,
								 &dw,
								 NULL,
								 NULL );
		
		assert (hPipe != INVALID_HANDLE_VALUE); 
		
		BOOL res;
				
		// Access the processes through InstanceListBuf
		for (ThisInstance = InstanceListBuf; *ThisInstance != 0 ; ThisInstance += lstrlen(ThisInstance) +1 ) 
		{					
			if( strcmp(ThisInstance,"Idle") != 0 && strcmp(ThisInstance,"_Total") != 0 
				&& strcmp(ThisInstance,"prosList") != 0) 
      {
				res = WriteFile( hPipe,
							     ThisInstance,
							     strlen(ThisInstance),
								 &nWritten,
								 NULL);									
				prosCount++;				
			}	
		}
	}	
	
	PdhCloseQuery(hQuery);
	CloseHandle(hPipe);
	
	return 0;
}



