
#include "cpuLoad.h"

#include "../../../Common/error.h"



DWORD CreatePipe(HANDLE &hPipe, LPSTR lpPipeName, int bufSize) 
{	
	STARTUPINFO	si;
	PROCESS_INFORMATION pi;
	BOOL		RetCode;
	DWORD		nRead=0;
	DWORD		dwErr;	
	CONST DWORD msgSize=1000;
	char		errMsg[1000];

	strcpy(errMsg, "Error in CreatePipe: %s");	

	hPipe = CreateNamedPipe( lpPipeName,	// pointer to pipe name
							 PIPE_ACCESS_INBOUND,	// The flow of data in the pipe goes 
													// from client to server only
							 PIPE_TYPE_MESSAGE |	// Data is written to the pipe as a 
													// stream of messages
							 PIPE_WAIT,	// blocking mode is enabled
							 1,			// maximum number of instances
							 bufSize,	// output buffer size, in bytes	
							 0,			// input buffer size, in bytes
							 0,			// time-out time, in milliseconds
							 NULL );	// pointer to security attributes
	dwErr = GetLastError();	
	if(dwErr != ERROR_SUCCESS) 
	{
		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
		return dwErr;
	}

	
	memset(&si, 0, sizeof(si));
	si.cb = sizeof(si);

  const char* processName = PROSLISTFILE;

  // TEMP : just for debugging
  if (g_Debug)
  {
    processName = DEBUG_PROSLISTFILE;
  }
  // TEMP

	RetCode = CreateProcess( processName,	// lpApplicationName
								NULL,		// lpCommandLine
								NULL,		// lpProcessAttributes
								NULL,		// lpThreadAttributes 
								FALSE,		// bInheritHandles
								0,			// dwCreationFlags
								NULL,		// lpEnvironment 
								NULL,		// lpCurrentDirectory
								&si,		// lpStartupInfo
								&pi	);		// lpProcessInformation
	dwErr = GetLastError();	
	if(dwErr != ERROR_SUCCESS) 
	{
		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
		return dwErr;
	}
	
	// NECESSARY!! we don't need them, so close 'em
	CloseHandle( pi.hThread );
	CloseHandle( pi.hProcess );	
	
	return ERROR_SUCCESS;
}
									

DWORD BuildProcessList(HQUERY &hQuery, struct ProcessInfo* prosInfoTab, int &prosCount) 
{							
	char		str[MAX_PROCESS_NAME_LEN], tmp[MAX_PROCESS_NAME_LEN];
	int			msgSize = MAX_PROCESS_NAME_LEN;	
	int			res;
	int			cnt=0;
	DWORD		BufSize = prosCount;
	PDH_STATUS	err;
	LPTSTR		CounterListBuf = NULL;
	DWORD		CounterListSize = 0;
	LPTSTR		InstanceListBuf = NULL;
	DWORD		InstanceListSize = 0;
	LPSTR		ThisInstance = NULL;
	HANDLE		hPipe;
	LPTSTR		lpPipeName = "\\\\.\\pipe\\ProsListPipe"; 
	DWORD		nRead=0;	

	PDH_COUNTER_PATH_ELEMENTS CounterPathElements;

	CounterPathElements.szMachineName = NULL;
	CounterPathElements.szObjectName = "Process";
	CounterPathElements.szParentInstance = NULL;
	CounterPathElements.szCounterName = "% Processor Time";
	
	if(hQuery!=NULL)
		PdhCloseQuery(hQuery); 
	
	prosCount = 0;

	err = PdhOpenQuery (0, 0, &hQuery);
			
	assert( err == ERROR_SUCCESS);
	if(err != ERROR_SUCCESS) 
	{
		int			_i;
		char		errMsg[512];
		_i  = sprintf(errMsg, "Error in BuildProcessList. ");
		_i += sprintf(errMsg + _i, "PdhOpenQuery: 0x%X", err);		
		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
				
		return err;
	}		
	
	err = CreatePipe(hPipe, lpPipeName, msgSize);
	
	assert( err == ERROR_SUCCESS);
	if(err != ERROR_SUCCESS) 
	{		
		printToLog(ERROR_LEVEL, CPULOG, "CreatePipe : %s", GetLastErrorText());
		return err;
	}	
	
	// Wait for client
	err = ConnectNamedPipe(hPipe, NULL);
  if (err == 0) // Error
  {
	  err = GetLastError();
  	if(err != ERROR_SUCCESS) 
	  {
		  int			_i;
		  char		errMsg[512];
      GetLastErrorText(errMsg, 512, err);
		  _i  = sprintf(errMsg, "ConnectNamedPipe in BuildProcessList");
		  _i += sprintf(errMsg + _i, "System error code: %d", err);
  		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
				  
		  CloseHandle(hPipe);
		  return err;
  	}
  }
	
	
	// Communicate
	do 
	{		
		err = ReadFile( hPipe,
						str,  
						300,
						&nRead,
						NULL);	
				
		if(nRead>0) 
		{				
			str[nRead]='\0';
			strcpy(prosInfoTab[prosCount].processName, str);			
			prosInfoTab[prosCount].instance = 0;			
			
			// Check if there are more instances of this process
			for(int i = 0; i < prosCount+1; i++) 
			{			
				res = strcmp(str, prosInfoTab[i].processName);
				if( res == 0 ) {
					cnt++;
					if(cnt > 1)
						prosInfoTab[prosCount].instance++;							
				}
			}
			cnt=0;
			
			// Build counter path	
			CounterPathElements.szInstanceName = str;
			CounterPathElements.dwInstanceIndex = prosInfoTab[prosCount].instance;
		
			err = PdhMakeCounterPath( &CounterPathElements,
									  tmp,
									  &BufSize,
									  0 );
		
			if(err != ERROR_SUCCESS) 
			{
				int			_i;
				char		errMsg[512];

				_i  = sprintf(errMsg, "PdhMakeCounterPath in BuildProcessList. ");
				_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
				_i += sprintf(errMsg + _i, "Process %s will not be added to the process list", prosInfoTab[prosCount].processName);				
    		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
				
				continue;				
			}						

			err = PdhAddCounter( hQuery, tmp, 0, &prosInfoTab[prosCount].hCounter);
			
			assert( err == ERROR_SUCCESS);
			if(err != ERROR_SUCCESS) 
			{
				int			_i;
				char		errMsg[512];

				_i  = sprintf(errMsg, "PdhAddCounter in BuildProcessList. ");
				_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
				_i += sprintf(errMsg + _i, "Could not add process %s to the process list", prosInfoTab[prosCount].processName);				
		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
				
				continue;
			}

			prosCount++;
		}		
	}
	while(nRead > 0);
	
	// update, necessary to make the counter value valid		
	err = PdhCollectQueryData(hQuery);	
	assert( err == ERROR_SUCCESS);		
			
	CloseHandle(hPipe);	
	
	return err;
}


DWORD WINAPI CollectCpuLoad(LPVOID lpvParam) 
{	
	HQUERY		hCPUQuery = NULL; 
	HCOUNTER	hCpuCounter = NULL;	

	HQUERY		hProsQuery = NULL; 
	HCOUNTER	hProcessCounter = NULL;

	DWORD					ctrType;
	PDH_FMT_COUNTERVALUE	fmtValue;
	
	PDH_STATUS	err = ERROR_SUCCESS;
	
	LPTSTR		CounterListBuf = NULL;
	DWORD		CounterListSize = 0;
	LPTSTR		InstanceListBuf = NULL;
	DWORD		InstanceListSize = 0;
	int			prosCount = 0;	
	int			last = 0;
	bool		firstTry = true;
	int			max = 0;	
	int			cnt = 0;	
	
	char		errMsg[512];
	int			_i;

	char		cpuTotCounterPath[256];
	DWORD		version;
	DWORD		majorVersion;
	DWORD		minorVersion;

	HANDLE		hCurPros;

	// tell our parent that we are running
	cpuThrdIsRunning = true;
	
	// get Windows-version
	version = GetVersion();
	majorVersion = (DWORD) (LOBYTE(LOWORD(version)));
	minorVersion = (DWORD) (HIBYTE(LOWORD(version)));

	if(majorVersion<5) // NT4
		strcpy(cpuTotCounterPath,"\\System\\% Total Processor Time");
	else // W2K
		strcpy(cpuTotCounterPath,"\\Processor(_Total)\\% Processor Time");	

	for(int i=0; i < CPULoadArraySize; i++)
		CPU.CpuLoadLast_15_Min[i] = 0;
	
	CPU.CPULoadArrayIndex = 0;	

	Process.cpuLoad = 0;	
	
	struct ProcessInfo* prosInfoTab;

	err = PdhOpenQuery (0, 0, &hCPUQuery);	
	
	assert( err == ERROR_SUCCESS);
	if(err != ERROR_SUCCESS) 
	{		
		_i  = sprintf(errMsg, "PdhOpenQuery(0,0,&hCPUQuery) in CollectCpuLoad. ");
		_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
		_i += sprintf(errMsg + _i, "Cannot continue. Cpu thread exits");
		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
				
		goto cleanup;
	}	

	// Counter for TOTAL CPU-load
	err = PdhAddCounter( hCPUQuery, cpuTotCounterPath, 0,  &hCpuCounter);
	
	assert( err == ERROR_SUCCESS);
	if(err != ERROR_SUCCESS) 
	{
		_i  = sprintf(errMsg, "PdhAddCounter in CollectCpuLoad. ");
		_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
		_i += sprintf(errMsg + _i, "Failed to add cpu-load counter. Cpu thread exits");
		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
	
		goto cleanup;
	}	
	
	// Counter for number of processes running on the system
	err = PdhAddCounter( hCPUQuery, "\\Objects\\Processes", 0,  &hProcessCounter) ;
	
	assert( err == ERROR_SUCCESS);
	if(err != ERROR_SUCCESS) 
	{
		_i  = sprintf(errMsg, "PdhAddCounter in CollectCpuLoad. ");
		_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
		_i += sprintf(errMsg + _i, "Failed to add process-count counter. Cpu thread exits");
		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
		
		goto cleanup;
	}
			
	while(1) 
	{
		// collect data: cpu-load and number of processes
		err = PdhCollectQueryData(hCPUQuery);
		
		assert( err == ERROR_SUCCESS);
		if(err != ERROR_SUCCESS) 
		{
			_i  = sprintf(errMsg, "PdhCollectQueryData in CollectCpuLoad. ");
			_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
			_i += sprintf(errMsg + _i, "Failed to collect cpu-load data. Cpu thread exits");
  		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());

			break;
		}		
		
		// number of processes running on the system
		fmtValue.longValue = 0;
		err = PdhGetFormattedCounterValue( hProcessCounter, PDH_FMT_LONG, &ctrType, &fmtValue);		
		
		assert( err == ERROR_SUCCESS);
		if(err != ERROR_SUCCESS) 
		{
			_i  = sprintf(errMsg, "PdhGetFormattedCounterValue in CollectCpuLoad. ");
			_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
			_i += sprintf(errMsg + _i, "Failed to get number of process running on the system. Cpu thread exits");
  		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());

			break;
		}		
				
		// if the process count changes, build the process list again
		if( firstTry | (fmtValue.longValue != last) ) 
		{			
			int i=0;		
			if( !firstTry )
				delete[] prosInfoTab;						

			prosCount = fmtValue.longValue;
			prosInfoTab = new struct ProcessInfo[prosCount];		
					
			err = BuildProcessList(hProsQuery, prosInfoTab, prosCount);
			
			assert( err == ERROR_SUCCESS);		
			if(err != ERROR_SUCCESS) 
			{
				_i  = sprintf(errMsg, "Failed to build process list. ");
				_i += sprintf(errMsg + _i, "Cannot continue. Cpu thread exits");						
    		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
				break;
			}			
						
			firstTry=false;
			last = fmtValue.longValue;	

			// remove as many pages as possible from the working set			
			hCurPros = GetCurrentProcess();
			SetProcessWorkingSetSize( hCurPros, 0xFFFFFFFF , 0xFFFFFFFF );
			CloseHandle(hCurPros);
		}
		else
			last = fmtValue.longValue;
		

		
		// Collect CPU-data for all the processes on the system
		err = PdhCollectQueryData(hProsQuery);
		
		assert( err == ERROR_SUCCESS);
		if(err != ERROR_SUCCESS) 
		{
			_i  = sprintf(errMsg, "PdhCollectQueryData in CollectCpuLoad. ");
			_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
			_i += sprintf(errMsg + _i, "Failed to collect cpu-load data for the processes running on the system. Cpu thread exits");
  		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());

			break;
		}		
			
		err = PdhGetFormattedCounterValue( hCpuCounter, PDH_FMT_LONG, &ctrType, &fmtValue);
		
		assert( err == ERROR_SUCCESS);
		if(err != ERROR_SUCCESS) 
		{
			_i  = sprintf(errMsg, "PdhGetFormattedCounterValue in CollectCpuLoad. ");
			_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
			_i += sprintf(errMsg + _i, "Failed to get cpu-load for the processor(s). Cpu thread exits");
  		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());

			break;
		}					

		// Update the arrary
		EnterCriticalSection(&ProcessAccess);

		CPU.CpuLoadLast_15_Min[CPU.CPULoadArrayIndex] = (short) fmtValue.longValue;
		CPU.CPULoadArrayIndex = (CPU.CPULoadArrayIndex + 1) % CPULoadArraySize;

		// Find the process with highest CPU-load
		max = 0;
		prosInfoTab[0].cpuLoad = max;
		Process = prosInfoTab[0];		
		
		for(int i=0; i < prosCount; i++) 
		{			
			err = PdhGetFormattedCounterValue( prosInfoTab[i].hCounter, PDH_FMT_LONG, &ctrType, &fmtValue);
			if( err != ERROR_SUCCESS &&
          strcmp(prosInfoTab[i].processName, "eap") != 0)
			{
        char lastError[512];
        GetLastErrorText(lastError, 512, err);

				_i  = sprintf(errMsg, "PdhGetFormattedCounterValue in CollectCpuLoad. ");
				_i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
				_i += sprintf(errMsg + _i, "Failed to get cpu-load from the process: %s", prosInfoTab[i].processName);
    		printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
				
				// try to build the process list again 
				last = 0;
				continue;
			}	
					
			prosInfoTab[i].cpuLoad = (short) fmtValue.longValue;
			

			if( fmtValue.longValue > max) 
			{
				max = fmtValue.longValue;								
				Process = prosInfoTab[i];				
			}			
		}

		LeaveCriticalSection(&ProcessAccess);		
		
		Sleep(SAMPLE_INTERVAL);	
	}
	
cleanup:
	// tell that we are stopped
	cpuThrdIsRunning = false;

	if(hProsQuery!=NULL)
		PdhCloseQuery(hProsQuery);
	if(hCPUQuery!=NULL)
		PdhCloseQuery(hCPUQuery);
	
	return err;
}