#ifndef _cpuLoadH
#define _cpuLoadH

#include <assert.h>
#include <PDH.h>
#include <iostream.h>
#include <fstream.h>
#include "../../../Common/error.h"
#include "eventLog.h"
#include <stdio.h>

#include "common.h"
#include "checks.h"



//#define	ENUMPROCFILE			"..\\eap.exe"	// process which generates list of all process
//#define	ENUMPROCFILE			"eap.exe"	// process which generates list of all process
											// running on the system
#define SAMPLE_INTERVAL			1000 // milliseconds
#define MAX_PROCESS_NAME_LEN	300
#define CPULoadArraySize		900



//
//  FUNCTION: int CreatePipe(HANDLE &hPipe, LPSTR lpPipeName, int bufSize)
//
//  PURPOSE: Create a communication channel between this process and the process
//			 referenced by lpPipeName.
//
//  PARAMETERS:
//		hPipe - If the function succeeds, hPipe is a handle 
//				to the server end of a named pipe instance. 
//				If the function fails, hPipe = INVALID_HANDLE_VALUE
//		lpPipeName - Pointer to the null-terminated string that uniquely 
//					 identifies the pipe
//		bufSize - Specifies the number of bytes to reserve for the output buffer
//		
//    
//  RETURN VALUES:
//		ERROR_SUCCESS - Everything ok
//		Call GetLastError() to get extended error information
//
DWORD CreatePipe(HANDLE &hPipe, LPSTR lpPipeName, int bufSize);


//
//  FUNCTION: DWORD BuildProcessList(HQUERY &hQuery, struct ProcessInfo* prosInfoTab, int &prosCount)
//
//  PURPOSE: Build an array of all process running on the system and add a 
//			 performance counter for each process.
//
//  PARAMETERS:
//		hQuery - Handle of the query to which the new counter will be added
//		prosInfoTab - the array to which the processes will be added
//		prosCount - number of processes which are added to the array
//
//  RETURN VALUES:
//		ERROR_SUCCESS - Everything ok
//		errors: pdh error codes or system error codes
//		
DWORD BuildProcessList(HQUERY &hQuery, struct ProcessInfo* prosInfoTab, int &prosCount);


//
//  FUNCTION: DWORD WINAPI CollectCpuLoad(LPVOID lpvParam)
//
//  PURPOSE: Thread-function which collects cpu-load values every second	
//	
DWORD WINAPI CollectCpuLoad(LPVOID lpvParam);

struct ProcessInfo 
{
	HCOUNTER	hCounter;
	int			cpuLoad;
	char		processName[MAX_PROCESS_NAME_LEN];	
	DWORD		instance;	
};

struct CPUInfo 
{	
	short 	CpuLoadLast_15_Min[CPULoadArraySize];
	int		CPULoadArrayIndex;
	
	double ComputeAvgCPULoad(int sek) {
		double CPULoadAvg = 0;
		int tmp = sek;
		int index = CPULoadArrayIndex-1;
		
		if(sek==0)
			return CpuLoadLast_15_Min[index];

		if(sek > CPULoadArraySize)
			return -1;		// ERROR!	
		
		while(tmp > 0) 
		{				
				
			CPULoadAvg += CpuLoadLast_15_Min[index--];
			
			if(index < 0)
				index = CPULoadArraySize-1;

			tmp--;			
		}		
		
		CPULoadAvg = CPULoadAvg / sek;		
		
		return CPULoadAvg;
	}
};

#endif