

#include "disks.h"

#include "../../../Common/error.h"
#include "../../../Common/queries.h"





bool getDiskSpace(const char* disk_name, ULARGE_INTEGER& all_disk, ULARGE_INTEGER& avail_disk)
{
	int err;
	ULARGE_INTEGER temp_disk_caller;
	ULARGE_INTEGER temp_disk_space;
	ULARGE_INTEGER temp_disk_avail;

	debug("Finding available disk space on: '%s'\n", disk_name);

	err = GetDiskFreeSpaceEx(disk_name, &temp_disk_caller, &temp_disk_space, &temp_disk_avail);
	if(!err)
  {
		printToLog(ERROR_LEVEL, ERRLOG, "GetDiskFreeSpaceEx() returned an error : %s!\n", GetLastErrorText() );
		return false;
	}
	
	// convert to KB and copy out
  all_disk.QuadPart = temp_disk_space.QuadPart;
	avail_disk.QuadPart = temp_disk_avail.QuadPart;
	return true;
}




int CheckDiskUsage( const KVBuffer& rbuf,
                    KVBuffer& sbuf)
{
  // Get Options
  const char* path = rbuf.getValue("path");

  // get requested information
  EnterCriticalSection(&ProcessAccess);


  // Real memory
  ULARGE_INTEGER all_disk;
  ULARGE_INTEGER avail_disk;
  if (!getDiskSpace(path, all_disk, avail_disk))
  {
    return ERR_GET_DISK_SPACE;
  }
  
  int free_mb = (int)(avail_disk.QuadPart >> 20);
  int total_mb = (int)(all_disk.QuadPart >> 20);
  

  sbuf.reset();

  sbuf.addKeyIntValue("id", DISKUSAGE_QUERY);
  sbuf.addKeyIntValue("free", free_mb);
  sbuf.addKeyIntValue("total", total_mb);
  
  
  LeaveCriticalSection(&ProcessAccess);	

  return NOERR;
}

