/*******************************************************************
*
* check_ntdisk plugin
* Copyright (c) 2000 Jan Christian Kaldestad
*
*
* Description:
* 	 Checks the processor load average on NT servers.
*	 Also finds the process with highest cpu usage.
*
******************************************************************/





#include "util.h"
#include "../../../Common/kvbuffer.h"
#include "../../../Common/error.h"
#include "../../../Common/queries.h"




int CheckCmdLine(int argc, char **argv, KVBuffer& buffer)
{
  if (argc != 7)
  {
    cout << "Incorrect arguments supplied\n\n";
    cout << "Usage: " << argv[0] << " <IPaddress> <port> <PERCENT|SIZE> <path> <wThresh> <cThresh>\n\n";
    cout << "Note:\n";
    cout << " <PERCENT|SIZE> = say if the limits are in PERCENT or MB" << endl;
    cout << " <path>      = full path to disk (or directory) to monitor" << endl;
    cout << " <wThresh>   = value of free space that result in a WARNING message," << endl;
    cout << " <cThresh>     = value of free space that result in a CRITICAL message," << endl;

    return STATE_UNKNOWN;
  }

  char path[128];
  strncpy(path, argv[4], 128);
  debug("Path : %s", path);

  char id[10];
  sprintf(id, "%d", DISKUSAGE_QUERY);

  buffer.addKeyValue("id", id);
  buffer.addKeyValue("path", path);

  return STATE_OK;
}


bool GetStringAndState(int argc, char** argv, KVBuffer& sbuffer, KVBuffer& rbuffer, int& state, char* msg)
{
  int free_mb = rbuffer.getIntValue("free");
  int total_mb = rbuffer.getIntValue("total");

  int wThreshold = atoi(argv[5]);
  int cThreshold = atoi(argv[6]);

  debug("free: %d / Total: %d / wThresh: %d / cThresh: %d", free_mb, total_mb, wThreshold, cThreshold);

  state = STATE_OK;
  const char* path = sbuffer.getValue("path");

  int size_or_percent = ( strcmp(argv[3], "SIZE") == 0 ? SIZE : PERCENT );

  if (size_or_percent == PERCENT)
  {
    int p = (int)(free_mb / (double)total_mb * 100);
    sprintf(msg, "Only %d%s free on %s (%d Mb / %d Mb)", p, "%%", path, free_mb, total_mb);
    debug("Msg: %s", msg);
    if (p > cThreshold)
    {
      state = STATE_CRITICAL;
    }
    else if (p > wThreshold)
    {
      state = STATE_WARNING;
    }
    else
    {
      sprintf(msg, "%d%s free on %s (%d Mb / %d Mb)", p, "%%", path, free_mb, total_mb);
    }
  }
  else if (size_or_percent == SIZE)
  {
    sprintf(msg, "Only %dMb free on %s (Total = %d Mb)", free_mb, path, total_mb);

    int p = free_mb;
    if (p < cThreshold)
    {
      state = STATE_CRITICAL;
    }
    else if (p < wThreshold)
    {
      state = STATE_WARNING;
    }
    else
    {
      sprintf(msg, "%dMb free on %s (Total = %d Mb)", free_mb, path, total_mb);
      state = STATE_OK;
    }
  }
  else
  {
    state = STATE_UNKNOWN;
    sprintf(msg, "Unknown option !");
  }

  return true;
}
