/*******************************************************************
*
* check_ntyourself plugin
* Copyright (c) 2000 Jan Christian Kaldestad 
*
*
* Description:
* 	 Checks the processor load average on NT servers.  
*	 Also finds the process with highest cpu usage.
*
******************************************************************/





#include "util.h"
#include "../../../Common/kvbuffer.h"
#include "../../../Common/error.h"
#include "../../../Common/queries.h"




int CheckCmdLine(int argc, char **argv, KVBuffer& buffer)
{
  if (argc != 5)
  {
    cout << "Incorrect arguments supplied\n\n";		
    cout << "Usage: " << argv[0] << " <IPaddress> <port> <wThresh> <cThresh>\n\n";
    cout << "Note:\n";
    cout << " <wThresh>      = value of free memory that result in a WARNING message," << endl;
    cout << " <cThresh>      = value of free memory that result in a CRITICAL message," << endl;
    
    return STATE_UNKNOWN;
  }

  char id[10];
  sprintf(id, "%d", SELFUSAGE_QUERY);
  buffer.addKeyValue("id", id);

  return STATE_OK;
}


bool GetStringAndState(int argc, char** argv, KVBuffer& sbuffer, KVBuffer& rbuffer, int& state, char* msg)
{
  int used = rbuffer.getIntValue("used");
  int initial = rbuffer.getIntValue("initial");

  int wThreshold = atoi(argv[3]);
  int cThreshold = atoi(argv[4]);

  int times = (int)((used * 10) / (double)initial);

  debug("Used: %d / Initial: %d / Times: %d / wThresh: %d / cThresh: %d", used, initial, times, wThreshold, cThreshold);

  state = STATE_OK;

  sprintf(msg, "NSService takes %1.1f times mem it used at begin (now=%2.1fMb / init=%2.1fMb)", (double)((double)times / 10), (double)((double)used/1024), (double)((double)initial/1024) );
  debug("Msg: %s", msg);
  if (times > cThreshold)
  {
    state = STATE_CRITICAL;
  }
  else if (times > wThreshold)
  {
    state = STATE_WARNING;
  }

  return true;
}
